--[[
Author: Mantrid
https://fs.mantrid.net/
Copyright: All rights reserved
LEGAL NOTICE: This mod, and this file in particular, are only authorised for distribution by GIANTS Software. All other entities agree to pay the sum of AU$1,000,000 for distribution rights.
Aspiring developers: I have limited experience with coding LUA, so my methods may not be the best. You're welcome to read and play with the code, for the purposes of learning, but do not copy whole functions for your own projects.
Experienced developers: It's my first day :)
]]

superPerson = {}

superPerson.buttonId = {}

function superPerson:loadMap(savegame)
	Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, superPerson.updateControlsMenu)
	Enterable.onRegisterActionEvents = Utils.appendedFunction(Enterable.onRegisterActionEvents, superPerson.updateControlsMenu)
	BaseMission.onLeaveVehicle = Utils.appendedFunction(BaseMission.onLeaveVehicle, superPerson.updateControlsMenu)
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, superPerson.removeActionEvents)

	Platform.gameplay.timeScaleSettings = { 0.5, 1, 2, 3, 5, 6, 10, 15, 30, 60, 120, 240, 360, 480, 600, 900, 1800, 3600, 5000, 10000, 20000, 30000 }

	Logging.info("Super Person mod has been loaded")
end

function superPerson:callbackMenu_main(what, axisComponent)
	if what == "SUPER_PERSON_TOGGLEHUD" then
		if g_currentMission.hud.isVisible then
			g_currentMission.hud:setIsVisible(false)
		else
			g_currentMission.hud:setIsVisible(true)
		end
	elseif what == "SUPER_PERSON_TOGGLEFLIGHT" then
		Player.consoleCommandToggleFlightMode(g_currentMission.player)
		Player.onInputDebugFlyToggle(g_currentMission.player)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_RUNSPEED" and axisComponent == -1 then
		g_currentMission.player.motionInformation.maxRunningSpeed = MathUtil.clamp(g_currentMission.player.motionInformation.maxRunningSpeed - 5, 2, 200)
		g_currentMission.player.motionInformation.maxCheatRunningSpeed = MathUtil.clamp(g_currentMission.player.motionInformation.maxCheatRunningSpeed - 5, 2, 200)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_RUNSPEED" and axisComponent == 1 then
		g_currentMission.player.motionInformation.maxRunningSpeed = MathUtil.clamp(g_currentMission.player.motionInformation.maxRunningSpeed + 5, 2, 200)
		g_currentMission.player.motionInformation.maxCheatRunningSpeed = MathUtil.clamp(g_currentMission.player.motionInformation.maxCheatRunningSpeed + 5, 2, 200)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_JUMPHEIGHT" and axisComponent == -1 then
		g_currentMission.player.motionInformation.jumpHeight = MathUtil.clamp(g_currentMission.player.motionInformation.jumpHeight - 1, 2, 50)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_JUMPHEIGHT" and axisComponent == 1 then
		g_currentMission.player.motionInformation.jumpHeight = MathUtil.clamp(g_currentMission.player.motionInformation.jumpHeight + 1, 2, 50)
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_SUPERSTRENGTH" then
		Player.consoleCommandToggleSuperStrongMode(g_currentMission.player)
		Player.MAX_PICKABLE_OBJECT_MASS = 500 -- Super Person can do more than 50!
		superPerson:updateControlsMenu()
	elseif what == "SUPER_PERSON_FLIGHTHELP" then
	else
		Logging.warning("Unknown menuCallback: " .. tostring(what))
	end
end

function superPerson:updateControlsMenu()
	-- Remove all of our actions
	for key, eventId in pairs(superPerson.buttonId) do
		g_inputBinding:removeActionEvent(eventId)
		superPerson.buttonId[key] = nil
	end

	local textEnabled = g_i18n:getText("fieldJob_active")
	local textDisabled = g_i18n:getText("ui_production_status_inactive")

	local stateFlightMode = textDisabled
	local stateSuperStrength = textDisabled
	local stateRunningSpeed
	if g_addCheatCommands then
		stateRunningSpeed = g_currentMission.player.motionInformation.maxCheatRunningSpeed
	else
		stateRunningSpeed = g_currentMission.player.motionInformation.maxRunningSpeed
	end

	if g_flightModeEnabled then stateFlightMode = textEnabled end
	if g_currentMission.player.superStrengthEnabled then stateSuperStrength = textEnabled end

	local actions = {
		{ "SUPER_PERSON_TOGGLEHUD", "callbackMenu_main", g_i18n:getText("superPerson_menu_toggleHUD"), 5 }
	}

	if g_currentMission.controlledVehicle == nil then
		table.insert(actions, { "SUPER_PERSON_TOGGLEFLIGHT", "callbackMenu_main", g_i18n:getText("superPerson_menu_toggleFlight") .. " (" .. stateFlightMode .. ")", 6 })
		table.insert(actions, { "SUPER_PERSON_RUNSPEED", "callbackMenu_main", g_i18n:getText("superPerson_menu_runSpeed") .. " (" .. stateRunningSpeed .. ")", 7 })
		table.insert(actions, { "SUPER_PERSON_JUMPHEIGHT", "callbackMenu_main", g_i18n:getText("superPerson_menu_jumpHeight") .. " (" .. g_currentMission.player.motionInformation.jumpHeight .. ")", 7 })
		table.insert(actions, { "SUPER_PERSON_SUPERSTRENGTH", "callbackMenu_main", g_i18n:getText("superPerson_menu_superStrength") .. " (" .. stateSuperStrength .. ")", 6 })
		if g_flightModeEnabled then
			table.insert(actions, { "SUPER_PERSON_FLIGHTHELP", "callbackMenu_main", g_i18n:getText("superPerson_menu_flightHelp"), 7 })
		end
	end

	for _, item in ipairs(actions) do
		local _, eventId = nil
		if superPerson.buttonId[item[3]] == nil then -- Skip if menu item already exists.. even though it shouldn't
			_, eventId = g_inputBinding:registerActionEvent(InputAction[item[1]], superPerson, superPerson[item[2]], false, true, false, true)
			superPerson.buttonId[item[3]] = eventId
			g_inputBinding:setActionEventActive(eventId, true)
			g_inputBinding:setActionEventText(eventId, item[3])
			g_inputBinding:setActionEventTextPriority(eventId, item[4])
			g_inputBinding:setActionEventTextVisibility(eventId, true)
		end
	end
end

function superPerson:removeActionEvents()
	local g_inputBinding = g_inputBinding
	-- Remove all of our actions
	for key, eventId in pairs(superPerson.buttonId) do
		g_inputBinding:removeActionEvent(eventId)
		superPerson.buttonId[key] = nil
	end
end

addModEventListener(superPerson)